#include "hashmap.h"
#include "myhashmap.h"
#include <vector>
#include <unordered_map>
#include <time.h>
#include <stdio.h>

struct HashInt
{
  size_t operator()( int k ) const
  {
    size_t h = *(unsigned int *)&k;     // breaks strict-aliasing
    h ^= h >> 16;
    h *= 0x85ebca6b;
    h ^= h >> 13;
    h *= 0xc2b2ae35;
    h ^= h >> 16;
    return h;
  }
};

double time()
{
  return (double)clock() / CLOCKS_PER_SEC;
}

int main()
{
  HashMap<int, int> m0;
  std::unordered_map<int, int, HashInt> m1;
  myhashmap::HashMap<int, int> m2;
  std::vector<int> keys(4000);
  for (size_t i = 0; i < keys.size(); i++) {
    keys[i] = (rand() ^ (rand()<<15)) / 10000;	// xor because microsoft stdlib uses 15 bits only
    m0.insert(keys[i], 1);
    m1[keys[i]] = 1;
    m2.Insert(keys[i], 1);
  }
  printf("m0.size() == m1.size() -> %s\n",
         m0.size() == (int)m1.size() ? "yes" : "no!!");
  printf("m0.size() == m2.size() -> %s\n",
    m0.size() == (int)m2.GetSize() ? "yes" : "no!!");
  double t0 = time();
  int found = 0;
  for (int j = 0; j < 50000; j++) {
    for (size_t i = 0; i < keys.size(); i++) {
      if (m0.find(keys[i])) {
        found++;
      }
      if (m0.find(keys[i] * 2)) {    // pravdepodobne == 0
        found++;
      }
    }
  }
  printf("%gs, found=%d\n", time() - t0, found);

  t0 = time();
  found = 0;
  for (int j = 0; j < 50000; j++) {
    for (size_t i = 0; i < keys.size(); i++) {
      if (m1.find(keys[i]) != m1.end()) {
        found++;
      }
      if (m1.find(keys[i] * 2) != m1.end()) {   // pravdepodobne == 0
        found++;
      }
    }
  }
  printf("%gs, found=%d\n", time() - t0, found);

  t0 = time();
  found = 0;
  for (int j = 0; j < 50000; j++) {
    for (size_t i = 0; i < keys.size(); i++) {
      if (m2.Find(keys[i]) != m2.End()) {
        found++;
      }
      if (m2.Find(keys[i] * 2) != m2.End()) {   // pravdepodobne == 0
        found++;
      }
    }
  }
  printf("%gs, found=%d\n", time() - t0, found);
  return 0;
}
